// Mail.cpp: implementation of the CMail class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Postman.h"
#include "Mail.h"
#include "MainFrm.h"
#include "States.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CMail::CMail()
{
	pFilePointer = NULL;
	Valid = TRUE;
	Downloaded = TRUE;
	Error = FALSE;

}

CMail::~CMail()
{

}

CMailList::CMailList(CMainFrame *pFW):ReceiveDlg( TRUE ), SendDlg( FALSE)
{
	pFrameWindow = pFW;
	Start = 0;
	pPOPSocket = NULL;
	pSMTPSocket = NULL;
	ImageIdx = 0;
	ReceiveDlg.SetMailList( this );
	SendDlg.SetMailList( this );
	WaitingForAuthentication = FALSE;
	WaitingForPOPQuiting = FALSE;
}

CMailList::~CMailList()
{
	if( pPOPSocket && pPOPSocket->State != NOTCONNECTED )
		pPOPSocket->Close();
	if( pSMTPSocket && pSMTPSocket->State != NOTCONNECTED )
		pSMTPSocket->Close();
}

CMail * CMailList::AddMail(DWORD No)
{
	CMail *NewMail;
	for( NewMail = Start; NewMail != NULL; NewMail = NewMail->Next )
	{
		if( NewMail->No == No )
			return NULL;
	}
	NewMail = new CMail;
	NewMail->Next = Start;
	Start = NewMail;
	NewMail->No = No;
	return NewMail;
}

CMail * CMailList::AddMail()
{
	DWORD No = 0;
	CMail *NewMail;

	for( NewMail = Start; NewMail != NULL; NewMail = NewMail->Next )
	{
		if( No < NewMail->No )
			No = NewMail->No;
	}
	return AddMail( No + 1 );
}



CMail* CMailList::GetMail( DWORD No)
{
	CMail *Mail;
	for( Mail = Start; Mail != NULL; Mail = Mail->Next )
	{
		if( Mail->No == No )
			return Mail;
	}
	return NULL;
}

void CMailList::ReceiveMail(CString & POPServer, CString & UserName, CString & Password)
{
	//Code to create a socket and download mail 
	if( pPOPSocket )
	{
		if( pPOPSocket->State == NOTCONNECTED )
			delete pPOPSocket;
		else
		{
			AfxMessageBox( "Socket is in use. Try after disconnection.");
			return;
		}
	}
	CreateReceiveDialogBox();
	pPOPSocket  = new CPOPSocket( this, &ReceiveDlg );
	pPOPSocket ->Create();
	pPOPSocket->m_UserName=UserName;
	pPOPSocket->m_Password=Password;
	pPOPSocket->m_POPServer = POPServer;

	CString Message = "Attempting Connection to " + POPServer + " ...";
	ReceiveDlg.SetHeader( Message );
	pPOPSocket->State = CONNECTING;
	if( pPOPSocket ->Connect(POPServer, 110) == SOCKET_ERROR ) 
	{
		int err1 = pPOPSocket ->GetLastError();
		switch (err1)
		{
			case WSANOTINITIALISED   :Message = "A successful AfxSocketInit must occur before using this API.";
			case WSAENETDOWN		: Message = "The Windows Sockets implementation detected that the network subsystem failed.";
			case WSAEAFNOSUPPORT   :Message = "The specified address family is not supported.";
			case WSAEINPROGRESS   :Message = "A blocking Windows Sockets operation is in progress.";
			case WSAEMFILE   :Message = "No more file descriptors are available.";
			case WSAENOBUFS   :Message = "No buffer space is available. The socket cannot be created.";
			case WSAEPROTONOSUPPORT   :Message = "The specified port is not supported.";
			case WSAEPROTOTYPE   :Message = "The specified port is the wrong type for this socket.";
			case WSAESOCKTNOSUPPORT   :Message = "The specified socket type is not supported in this address family.";
			default: Message = "Unable to connect";
		}
		ReceiveDlg.AddToStatus( Message );
		ReceiveDlg.m_Animate.Close();
		ReceiveDlg.m_Animate.ShowWindow(SW_HIDE); 
		pPOPSocket->State = NOTCONNECTED;
	}
	else
	{
		Message = "Connecting...";
		ReceiveDlg.SetHeader( Message );
	}
}

void CMailList::OnHeaderReceipt(DWORD No, BOOL Done )
{
	#ifdef _DEBUG
		pFrameWindow->AssertValid();
	#endif

	if( pFrameWindow )
		pFrameWindow->OnHeaderReceipt( No, Done);
}

void CMailList::OnMailReceipt(DWORD No)
{
	#ifdef _DEBUG
		pFrameWindow->AssertValid();
	#endif

	if( pFrameWindow )
		pFrameWindow->OnMailReceipt( No);
}

void CMailList::OnMailSent(DWORD No, BOOL Success)
{
	#ifdef _DEBUG
		pFrameWindow->AssertValid();
	#endif

	if( pFrameWindow )
		pFrameWindow->OnMailSent( No, Success );
	
	CMail *Mail;
	for( Mail = Start; Mail != NULL ; Mail = Mail->Next )
	{
		if( Mail->Valid  && !Mail->Error && 
			CPostmanApp::CompareAddresses( pFrameWindow->GetFromAddress(), Mail->From ) )
			break;
	}
	
	if( Mail )
	{
		ReadInMail( Mail->No );
		pSMTPSocket->SendMail( );
	}
	else
	{
		pSMTPSocket->Quit();
	}
}

void CMailList::SendMail(CString &SMTPServer)
{
	if( !GetCount() )
	{
		AfxMessageBox("No mail in outbox!");
		return;
	}
	if( !GetCount(pFrameWindow->GetFromAddress() ) )
	{
		AfxMessageBox("No mail in the outbox for this user!");
		return;
	}
	if( pSMTPSocket )
	{
		if( pSMTPSocket ->State == NOTCONNECTED )
			delete pSMTPSocket ;
		else
		{
			AfxMessageBox( "Socket is in use. Try after disconnection.");
			return;
		}
	}

	TotalSize = GetSize();
	
	CreateSendDialogBox();
	CString Message = "Attempting Connection to " + SMTPServer + " ...";
	SendDlg.SetHeader( Message );

	pSMTPSocket = new CSMTPSocket( this, &SendDlg );
	pSMTPSocket->Create();
	pSMTPSocket->State = CONNECTING;
	
	if( pSMTPSocket->Connect( SMTPServer, 25 ) == SOCKET_ERROR ) 
	{
		int err1 = pSMTPSocket->GetLastError();
		switch (err1)
		{
			case WSANOTINITIALISED   :Message = "A successful AfxSocketInit must occur before using this API.";
			case WSAENETDOWN		: Message = "The Windows Sockets implementation detected that the network subsystem failed.";
			case WSAEAFNOSUPPORT   :Message = "The specified address family is not supported.";
			case WSAEINPROGRESS   :Message = "A blocking Windows Sockets operation is in progress.";
			case WSAEMFILE   :Message = "No more file descriptors are available.";
			case WSAENOBUFS   :Message = "No buffer space is available. The socket cannot be created.";
			case WSAEPROTONOSUPPORT   :Message = "The specified port is not supported.";
			case WSAEPROTOTYPE   :Message = "The specified port is the wrong type for this socket.";
			case WSAESOCKTNOSUPPORT   :Message = "The specified socket type is not supported in this address family.";
			default: Message = "Unable to connect";
		}
		SendDlg.SetHeader( Message );
		SendDlg.m_Animate.Close();
		SendDlg.m_Animate.ShowWindow(SW_HIDE); 
	}
	else
	{
		Message = "Connecting...";
		SendDlg.SetHeader( Message );
		pSMTPSocket->TotalSize = TotalSize;
	}	
}

void CMailList::RemoveAll()
{
	CMail *Temp, *Prev;
	Temp = Start; 
	while( Temp )
	{
		Prev = Temp; 
		Temp = Temp->Next;
		delete Prev;
	}
	Start = NULL;
}

BOOL CMailList::ReadInMail()
{
	CMail *Mail;
	for( Mail = Start; Mail != NULL; Mail = Mail->Next )
	{
		if( Mail->Valid && CPostmanApp::CompareAddresses( Mail->From, pFrameWindow->GetFromAddress() ) )
			return ReadInMail( Mail->No );
	}
	return FALSE;
}

BOOL CMailList::ReadInMail( DWORD No )
{
	CMail *Mail = GetMail( No );
	if( !Mail )
	{
		return FALSE;
	}

	pSMTPSocket->m_Message.Empty();
	pSMTPSocket->m_BCC.Empty();
	pSMTPSocket->m_CC.Empty();
	pSMTPSocket->m_From.Empty();
	pSMTPSocket->m_To.Empty();

	CFile File;
	if( File.Open( Mail->FileName , CFile::modeRead ) )
	{
		DWORD nFileSize = File.GetLength();
		char * pData = new char[nFileSize + 1 ];
		BOOL Header = TRUE;
		File.Read(pData, nFileSize );
		pData[nFileSize] = 0;
		CString Line;
		
		for( DWORD i = 0; i < nFileSize ; i++ )
		{
			Line += pData[i];
			if( pData[i] == '\r' && pData[i+1] == '\n' )
			{
				i++;
				Line += pData[i];
				if( Header )
				{
					if( Line.GetLength() == 2 )
					{
						Header = FALSE;
					}
					else
					{
						if( ( Line[0] == 'S' || Line[0] == 's' )&& !Line.Left(7).CompareNoCase( "Subject" ) )
						{
							pSMTPSocket->m_Subject = Line.Right( Line.GetLength() - Line.Find(':') - 1 );
							pSMTPSocket->m_Subject = pSMTPSocket->m_Subject.Left(pSMTPSocket->m_Subject.GetLength()-2);
							pSMTPSocket->m_Subject.TrimLeft();
							pSMTPSocket->m_Subject.TrimRight();
						}
						else if( ( Line[0] == 'F' || Line[0] == 'f' ) && !Line.Left(4).CompareNoCase("From" ) )
						{
							pSMTPSocket->m_From = Line.Right( Line.GetLength() - Line.Find(':') - 1 );
							pSMTPSocket->m_From = pSMTPSocket->m_From.Left(pSMTPSocket->m_From.GetLength()-2);
							pSMTPSocket->m_From.TrimLeft();
							pSMTPSocket->m_From.TrimRight();
						}
						else if( !Line.Left(2).CompareNoCase("To" ) )
						{
							pSMTPSocket->m_To = Line.Right( Line.GetLength() - Line.Find(':') - 1 );
							pSMTPSocket->m_To = pSMTPSocket->m_To.Left(pSMTPSocket->m_To.GetLength()-2);
							pSMTPSocket->m_To.TrimLeft();
							pSMTPSocket->m_To.TrimRight();
						}
						else if( !Line.Left(2).CompareNoCase("CC" ) )
						{
							pSMTPSocket->m_CC = Line.Right( Line.GetLength() - Line.Find(':') - 1 );
							pSMTPSocket->m_CC = pSMTPSocket->m_CC.Left(pSMTPSocket->m_CC.GetLength()-2);
							pSMTPSocket->m_CC.TrimLeft();
							pSMTPSocket->m_CC.TrimRight();
						}
					}
				}
				pSMTPSocket->m_Message += Line;
				Line.Empty();
			}
		}
		//AfxMessageBox( pSMTPSocket->m_Message );
		pSMTPSocket->MailNo = Mail->No;
		free( pData );
		return TRUE;
	}
	else
		return FALSE;

}

BOOL CMailList::DeleteMail(DWORD No)
{

	CMail *Mail = GetMail( No );
	if( !Mail )
	{
		return FALSE;
	}
	CString FileName = Mail->FileName;
	Remove( No );
	_unlink ( FileName );
	CMixFile::DeleteFile( FileName );	
	return TRUE;
}

void CMailList::Remove(DWORD No)
{
	CMail *Temp, *Prev;
	for( Temp = Start, Prev = Start; Temp != NULL; Prev = Temp, Temp = Temp ->Next )
	{
		if( Temp->No == No )
		{
			//Deleting...
			Temp->Valid = FALSE;
			break;
		}
	}

}

WORD CMailList::GetCount()
{
	CMail *Mail;
	TotalSize = 0;
	WORD Count = 0;
	for( Mail = Start; Mail != NULL; Mail = Mail->Next )
	{
		if( Mail->Valid )
		{
			Count++;
			TotalSize += Mail->Size;
		}
	}
	return Count;
}

WORD CMailList::GetCount(CString From)
{
	CMail *Mail;
	TotalSize = 0;
	WORD Count = 0;
	for( Mail = Start; Mail != NULL; Mail = Mail->Next )
	{
		if( Mail->Valid && CPostmanApp::CompareAddresses( Mail->From, From ) )
		{
			Count++;
			TotalSize += Mail->Size;
		}
	}
	return Count;
}

DWORD CMailList::GetSize()
{
	CMail *Mail;
	DWORD Size = 0;
	for( Mail = Start; Mail != NULL; Mail = Mail->Next )
	{
		if( Mail->Valid )
		{
			Size += Mail->Size;
		}
	}
	return Size;
}

void CMailList::DeleteFromServer(DWORD No)
{
	CString Message;
	if( !pPOPSocket || pPOPSocket->State != IDLE )
		return;
	Message.Format("DELE %d\r\n", No);
	pPOPSocket->LastCommand = DELE;
	pPOPSocket->State = TRANSACTION;
	pPOPSocket->DeletionRequestNo = No;
	pPOPSocket->Send( (LPCSTR) Message, Message.GetLength() );
	Message = "C:" + Message;
	ReceiveDlg.AddToStatus( Message );	
	ReceiveDlg.SetHeader( "Deleting mail...");
	ReceiveDlg.m_Count.Empty();
	ReceiveDlg.UpdateData( FALSE );
}

void CMailList::OnMailDelete(DWORD No, BOOL Success)
{
	if( Success )
		Remove( No );
	#ifdef _DEBUG
		pFrameWindow->AssertValid();
	#endif
	pFrameWindow->OnMailDelete( No, Success);
}

BOOL CMailList::MoveTo(DWORD No, CString &Destination, CMailList *DestinationList)
{
	CMail *Mail = GetMail( No );
	if( !Mail )
		return FALSE;
	CString NewName;

	//1. Creating an entry in dest. mail list	
	CMail *NewMail = DestinationList->AddMail( );
	NewMail->BCC = Mail->BCC;
	NewMail->CC = Mail->CC;
	NewMail->Date = Mail->Date;
	NewMail->Downloaded = Mail->Downloaded;
	NewMail->From = Mail->From;
	NewMail->ID = Mail->ID;
	NewMail->Size = Mail->Size;
	NewMail->Subject = Mail->Subject;
	NewMail->To = Mail->To;
	NewMail->Valid = TRUE;

	
	// 2. Moving file to dest folder...
	CPostmanApp::GetTempFileName( NewName, Destination );

	rename( Mail->FileName, NewName );
	NewMail->FileName = NewName;

	//3. Writing to MIX file
	CMixFile MixData;
	MixData.UID = Mail->ID;
	MixData.From = Mail->From;
	MixData.To = Mail->To;
	MixData.Date = Mail->Date;
	MixData.Subject = Mail->Subject;
	MixData.FileName = NewName;
	MixData.Size = Mail->Size;
	MixData.Status = 0;
	MixData.WriteMIX( Destination + "Mail Index.mix" );

	//4. Deleting old entry in Mix file
	CMixFile::DeleteFile( Mail->FileName );
	
	//5. Removing entry from MailList
	Remove( No );
	return TRUE;
}

BOOL CMailList::ForceStop(BOOL POP)
{
	if( pPOPSocket )
	{
		if( pPOPSocket->State != NOTCONNECTED )
			pPOPSocket->ShutDown( 2 );
		pPOPSocket->State = NOTCONNECTED;
		pPOPSocket->HeaderList.UpdateHeaders( FALSE );
	}

	if( ! POP )
	{
		if( pSMTPSocket )
		{
			if( pSMTPSocket->State != NOTCONNECTED )
				pSMTPSocket->ShutDown( 2 );
			pSMTPSocket->State = NOTCONNECTED;

		}	
		WaitingForAuthentication = FALSE;
		WaitingForPOPQuiting = FALSE;
	}
	return TRUE;
}

BOOL CMailList::Stop(BOOL POP)
{
	if( pPOPSocket && pPOPSocket->State != NOTCONNECTED )
		pPOPSocket->Quit();
	if( !POP )
	{
		if( pSMTPSocket && pSMTPSocket->State != NOTCONNECTED )
			pSMTPSocket->Quit();
	}
	return TRUE;
}

void CMailList::GetAuthentication(CString &SMTP_Server, CString &POPServer, CString &UserName, CString &Password)
{
	//Code to create a socket and download mail 
/*	if( !GetCount() )
	{
		AfxMessageBox("No mail in the outbox!");
		return;
	}*/

	if( !GetCount(pFrameWindow->GetFromAddress() ) )
	{
		AfxMessageBox("No mail for this user in the outbox!");
		return;
	}

	if( pSMTPSocket && pSMTPSocket->State != NOTCONNECTED )
	{
			AfxMessageBox( "Socket is in use. Try after disconnection.");
			return;
	}

	SMTPServer = SMTP_Server;
	if( pPOPSocket )
	{
		if( pPOPSocket->State == NOTCONNECTED )
			delete pPOPSocket;
		else
		{
			AfxMessageBox( "Socket is in use. Try after disconnection.");
			return;
		}
	}
	WaitingForAuthentication = TRUE;

	CreateSendDialogBox();
	CString Message = "Attempting to get POP authentication from" + POPServer + " ...";
	SendDlg.SetHeader( Message );
	pPOPSocket  = new CPOPSocket( this, &SendDlg, TRUE );
	pPOPSocket ->Create();
	pPOPSocket->m_UserName=UserName;
	pPOPSocket->m_Password=Password;
	pPOPSocket->m_POPServer = POPServer;
	pPOPSocket->State = CONNECTING;

	if( pPOPSocket ->Connect(POPServer, 110) == SOCKET_ERROR ) 
	{
		int err1 = pPOPSocket ->GetLastError();
		switch (err1)
		{
			case WSANOTINITIALISED   :Message = "A successful AfxSocketInit must occur before using this API.";
			case WSAENETDOWN		: Message = "The Windows Sockets implementation detected that the network subsystem failed.";
			case WSAEAFNOSUPPORT   :Message = "The specified address family is not supported.";
			case WSAEINPROGRESS   :Message = "A blocking Windows Sockets operation is in progress.";
			case WSAEMFILE   :Message = "No more file descriptors are available.";
			case WSAENOBUFS   :Message = "No buffer space is available. The socket cannot be created.";
			case WSAEPROTONOSUPPORT   :Message = "The specified port is not supported.";
			case WSAEPROTOTYPE   :Message = "The specified port is the wrong type for this socket.";
			case WSAESOCKTNOSUPPORT   :Message = "The specified socket type is not supported in this address family.";
			default: Message = "Unable to connect";
		}
		SendDlg.AddToStatus( Message );
		SendDlg.m_Animate.Close();
		SendDlg.m_Animate.ShowWindow(SW_HIDE); 
		pPOPSocket->State = NOTCONNECTED;
	}
	else
	{
		Message = "Connecting...";
		SendDlg.SetHeader( Message );
	}
}

void CMailList::OnPOPAuthentication(BOOL Success)
{
	if( Success && WaitingForAuthentication )
	{
		/**************** To be deleted FOR DEBUGGING *************************
		WaitingForAuthentication = FALSE;
		if( pPOPSocket )
			pPOPSocket->Quit();
		/*****************************************/
		
		 SendMail( SMTPServer );
	}
	else
		pFrameWindow->OnPOPAuthentication( Success );
}

void CMailList::OnSMTPAuthentication(BOOL Success)
{
	if( WaitingForAuthentication )
	{
		WaitingForAuthentication = FALSE;
		if( pPOPSocket )
			pPOPSocket->Quit();
	}
	else
		pFrameWindow->OnSMTPAuthentication( Success );
}

void CMailList::OnPOPQuit()
{
//	delete pPOPSocket;
//	pPOPSocket = NULL;
	pFrameWindow->OnPOPQuit();
}

void CMailList::OnSMTPQuit()
{
//	delete pSMTPSocket;
//	pSMTPSocket = NULL;
	pFrameWindow->OnSMTPQuit();
}

void CMailList::CreateSendDialogBox()
{
	if( !IsWindow( SendDlg.m_hWnd ) )
		SendDlg.Create( IDD_RECEIVE, CWnd::GetDesktopWindow() );	
	SendDlg.ShowWindow( SW_SHOW );
	HICON m_hIcon = AfxGetApp()->LoadIcon( IDI_OUTBOX );
	SendDlg.SetIcon(m_hIcon, TRUE);			// Set big icon
	SendDlg.SetIcon(m_hIcon, FALSE);		// Set small icon
	SendDlg.SetWindowText( "Send Mail");
	SendDlg.m_Animate.Open(IDR_SEND);
	SendDlg.m_Animate.ShowWindow(SW_SHOW);	
	SendDlg.m_Animate.Play(0,-1,-1);	

}

void CMailList::CreateReceiveDialogBox()
{
	if( !IsWindow(ReceiveDlg.m_hWnd) )
		ReceiveDlg.Create( IDD_RECEIVE, CWnd::GetDesktopWindow() );	
	ReceiveDlg.ShowWindow( SW_SHOW );

	ReceiveDlg.m_Animate.Open( IDR_RECEIVE );
	ReceiveDlg.m_Animate.ShowWindow( SW_SHOW );	
	ReceiveDlg.m_Animate.Play(0,-1,-1);

}

CMail* CMailList::GetMail(CString &UID)
{
	CMail *Mail;
	for( Mail = Start; Mail != NULL; Mail = Mail->Next )
	{
		if( Mail->ID.Compare( UID )== 0 )
			return Mail;
	}
	return NULL;
}