// POPSocket.cpp : implementation file
//

#include "stdafx.h"
#include "Postman.h"
#include "POPSocket.h"
#include "MainFrm.h"
#include "Mail.h"
#include "States.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


////////////////////////////////////////////////////////////////////////////
// CPOPSocket

CPOPSocket::CPOPSocket(CMailList * pML, CSndRecvDlg *DlgBox, BOOL bOnlyAuthentication )
{
	DialogBox = DlgBox;
	pMailList = pML;
	State = NOTCONNECTED;
	NoOfMessages = 0;
	NoOfBytes = 0;
	LastCommand = 0;
	MailReceived = 0;
	UIDsReceived = 0;
	BytesReceived = 0;
	DeletionRequestNo = -1;
	OnlyAuthentication = bOnlyAuthentication;
	DontQuit = FALSE;
	WantToQuit = FALSE;
}

CPOPSocket::~CPOPSocket()
{
	if( State != NOTCONNECTED )
	{
		State = NOTCONNECTED;
		Close();
	}
}


// Do not edit the following lines, which are needed by ClassWizard.
#if 0
BEGIN_MESSAGE_MAP(CPOPSocket, CAsyncSocket)
	//{{AFX_MSG_MAP(CPOPSocket)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#endif	// 0

/////////////////////////////////////////////////////////////////////////////
// CPOPSocket member functions


void CPOPSocket::OnConnect(int nErrorCode) 
{
	
	DialogBox->SetHeader( "Connection established with server. Waiting for greeting.");
	State = CONNECTED;
	NoOfMessages = 0;
	NoOfBytes = 0;
	CAsyncSocket::OnConnect(nErrorCode);
}

void CPOPSocket::OnClose(int nErrorCode) 
{
	
	if( State == TRANSACTION && LastCommand == RETR )
	{
		fclose ( Mail->pFilePointer );
		_unlink( Mail->FileName ); 
	}
	if( !OnlyAuthentication )
	{
		DialogBox->SetHeader("Connection with POP server terminated.");
		DialogBox->FlashWindow(TRUE);
		MessageBeep( MB_ICONHAND );
	}
	else
	{
		DialogBox->SetHeader("Done with POP Authentication.");
	}
	State = NOTCONNECTED;
	NoOfMessages = 0;
	NoOfBytes = 0;
	DialogBox->m_Animate.Close();
	DialogBox->m_Animate.ShowWindow(SW_HIDE); 
	//CAsyncSocket::OnClose(nErrorCode);
	Close();
	if( OnlyAuthentication )
		pMailList->OnPOPAuthentication( FALSE );
	//AfxMessageBox("OnClose : Connection with POP server terminated.");
}

void CPOPSocket::OnReceive(int nErrorCode) 
{
	// TODO: Add your specialized code here and/or call the base class
	CString Message;
	CString NextDataLine;
	DWORD Argument = 0;
	long Length = 0;
	IOCtl(FIONREAD, &Argument);
	if( Argument )
	{
		{
			BYTE * pData;
			pData = (BYTE *)malloc( Argument + 1);
			Length = Receive( pData, Argument );
			if( Length == -1 )
				Length = 0;
			pData[Length] = 0;	//Adding a NULL character
			DataStore += pData;
			free (pData);
		}
		while( 1 )
		{
			//Extracting the next line if any...
			BOOL LineComplete = FALSE;
			NextDataLine.Empty();
			Length = DataStore.GetLength();
			for( int i = 0; i < Length; i++ )
			{
				NextDataLine += DataStore[i];
				if( i > 0 && DataStore[i] == '\n' && DataStore[i-1] == '\r' )
				{
					//Found a complete Line
					DataStore = DataStore.Right( Length - i - 1);
					LineComplete = TRUE;
					break;
				}
			}
			if( !LineComplete )
				break;
			Message = "POP3 S: " ;
			Message += NextDataLine;
			DialogBox->AddToStatus( Message );
			Length = NextDataLine.GetLength();

			if( State == CONNECTED )
			{
				if( IsOK( NextDataLine ) == 1)
				{
					//Received the startup greeting from the POP3 server
					//Now in Authorisation state
					//Sending a USER message
					DialogBox->SetHeader("Attempting to get authorisation.");
					DialogBox->UpdateData();
					Message = "USER " + m_UserName + "\r\n";
					Send( (LPCSTR) Message, Message.GetLength() );
					Message = "C:" + Message;
					DialogBox->AddToStatus( Message );
					State = AUTHORISATION1;
				}
				else
				{
					//Invalid Response
					ShutDown(receives);
					State = NOTCONNECTED;
					if( OnlyAuthentication )
						pMailList->OnPOPAuthentication(FALSE);
					
					
					DialogBox->SetHeader( NextDataLine );
					Message = "Invalid response from the server.\nServer says : " + NextDataLine;
					AfxMessageBox( Message );
					//DialogBox->AddToStatus( Message );
				}
			}
			else if ( State == AUTHORISATION1 )
			{
				if( IsOK( NextDataLine ) == 1 )
				{
					//User Name accepted

					// Now sending the password
					//::MessageBox( NULL, "Waiting", "Can I send the Password?", MB_OK);
					DialogBox->UpdateData();
					Message = "PASS " + m_Password + "\r\n";
					Send( (LPCSTR) Message, Message.GetLength() );
					Message = "C:" + Message;
					DialogBox->AddToStatus( "PASS ********");
					State = AUTHORISATION2;
					//Creating the folder to hold the mail...
					MailFolder = CPostmanApp::AppData + "Inbox";
					MailFolder = MailFolder + "\\" + m_POPServer + "\\" +m_UserName ;
					CPostmanApp::CreateFolder( MailFolder );
					// Reading in the previously downloaded headers...
					HeaderList.ReadHeaders( MailFolder );
				}
				else
				{
					//Invalid User Name (probably)
					//Close();
					ShutDown(receives);
					State = NOTCONNECTED;
					Length = NextDataLine.GetLength() - 4;
					if( Length > 0 )
						Message = NextDataLine.Right(Length );
					else
						Message = "Username rejected";
					DialogBox->SetHeader( Message );
					if( OnlyAuthentication )
						pMailList->OnPOPAuthentication(FALSE);
					AfxMessageBox( Message );
					//Should give the user a chance to retry...
				}
			}
			else if ( State == AUTHORISATION2 )
			{
				if( IsOK( NextDataLine ) == 1 )
				{
					//Password accepted
					// Now in Transaction State

					pMailList->OnPOPAuthentication(TRUE);
					if( OnlyAuthentication )
					{
						State = IDLE;
					}
					else
					{
						DialogBox->SetHeader("Checking if there are any messages");
						Message = "STAT\r\n";
						Send( (LPCSTR) Message, Message.GetLength() );
						Message = "C:" + Message;
						DialogBox->AddToStatus( Message );
						State = TRANSACTION;
						LastCommand = STAT;
					}
				}
				else 
				{
					//Invalid Password
					//Close();
					ShutDown(receives);
					State = NOTCONNECTED;
					Length = NextDataLine.GetLength() - 4;
					if( Length > 0 )
						Message = NextDataLine.Right(Length );
					else
						Message = "Invalid Password";
					DialogBox->SetHeader( Message );
					AfxMessageBox(Message);
					if( OnlyAuthentication )
						pMailList->OnPOPAuthentication(FALSE);
					//Should give the user a chance to retry...
				}	
			}
			else if ( State == TRANSACTION )
			{
				if( LastCommand == STAT )
				{
					if( IsOK( NextDataLine ) == 1)
					{
						// Status Received
						//Parsing STAT message 
						sscanf( (const char *)NextDataLine+4, "%ld %ld", &NoOfMessages, &NoOfBytes);
						//Retrieving first Message 
						if( NoOfMessages )
						{
	
							Message = "UIDL\r\n";
							Send( (LPCSTR) Message, Message.GetLength() );
							Message = "C: UIDL\r\n" ;
							DialogBox->AddToStatus( Message );
							LastCommand = UIDL;
							UIDsReceived = 0;

							Message.Format("%d messages (%ld Bytes) ",NoOfMessages, NoOfBytes );
							DialogBox->SetHeader(Message );
						}
						else
						{
							State = IDLE;
							AfxMessageBox("Sorry! No mail for you!");
							Quit();
						}
						DialogBox->m_Count.Format( "0 of %d Header(s) received", NoOfMessages );
						DialogBox->UpdateData( FALSE );
						
					}
					else
					{
						//Invalid Status message
						//Close();
						ShutDown(receives);
						State = NOTCONNECTED;
						Length = NextDataLine.GetLength() - 4;
						if( Length > 0 )
							Message = NextDataLine.Right(Length );
						else
							Message = "Invalid Password";
						DialogBox->SetHeader( Message );
						AfxMessageBox(Message);
						Message = "Comment: Invalid response";
						DialogBox->AddToStatus( Message );
					}
				}
				else if ( LastCommand == UIDL )
				{
					if( IsOK( NextDataLine ) == 1);// Do nothing
					else if( NoOfMessages && Length > 2 && 
						NextDataLine[2] =='\n' && 
						NextDataLine[1] =='\r' && 
						NextDataLine[0] =='.' )
					{
						// Done receiveing Identifiers.
						// Now about to ask for size...
						Message = "LIST\r\n";
						Send( (LPCSTR) Message, Message.GetLength() );
						Message = "C:" + Message;
						DialogBox->AddToStatus( Message );
						LastCommand = LIST;
					}
					else
					{
						// Unique ID Listing...
						// Create Mail instances...
						char UID[71];
						WORD No;
						UIDsReceived ++;
						Mail = pMailList->AddMail( UIDsReceived );
						sscanf( (LPCSTR) NextDataLine,"%d %s", &No, UID );
						Mail->ID = UID;
						
						// Mail->FileName = MailFolder + "\\" + Mail->ID;
						ProcessHeader( Mail );
					}
				}
				else if ( LastCommand == LIST ) 
				{
					if( IsOK( NextDataLine ) == 1);// Do nothing
					else if( NoOfMessages && Length > 2 && 
						NextDataLine[2] =='\n' && 
						NextDataLine[1] =='\r' && 
						NextDataLine[0] =='.' )
					{
						// Done receiveing Sizes.
						// Now about to ask for headers...

						DWORD NextHeaderNo = GetNextHeaderNo( 1 );
						if( !NextHeaderNo )
						{
							AfxMessageBox( "Mail headers ready.");
							// DialogBox->ShowWindow( SW_HIDE );
							DialogBox->m_Count.Empty();
							DialogBox->UpdateData( FALSE );
							DialogBox->SetHeader( "Waiting for your next command, Oh Master!");
							State = IDLE;
						}
						else
						{
							Message.Format("TOP %d 0\r\n", NextHeaderNo );
							Send( (LPCSTR) Message, Message.GetLength() );
							Message = "C:" + Message;
							DialogBox->AddToStatus( Message );
							LastCommand = TOP;
							MailReceived	 = 0;
							//HeadersReceived = 0;
							Mail = pMailList->GetMail( NextHeaderNo );
						}
					}
					else
					{
						// Size Listing...
						DWORD Size;
						WORD No;
						sscanf( (LPCSTR) NextDataLine,"%d %ld", &No, &Size );
						Mail = pMailList->GetMail( No );
						Mail->Size = Size;
						
					}
				}
				else if ( LastCommand == TOP )
				{
					Header += NextDataLine;
					if( NoOfMessages && Length > 2 && 
						NextDataLine[2] =='\n' && 
						NextDataLine[1] =='\r' && 
						NextDataLine[0] =='.' )
					{
						DWORD NextHeaderNo = GetNextHeaderNo( Mail->No+1 );
						if( NextHeaderNo )
						{
							//Get next header...
							Message.Format("TOP %d 0 \r\n", NextHeaderNo );
							Send( (LPCSTR) Message, Message.GetLength() );
							Message = "C:" + Message;
							DialogBox->AddToStatus( Message );
							pMailList->OnHeaderReceipt( Mail->No );
						}
						else
						{
							State = IDLE;
							pMailList->OnHeaderReceipt( Mail->No , TRUE);
						}
						DialogBox->m_Progress.SetPos( (NextHeaderNo -1)* 100 / NoOfMessages);
						HeaderList.AddHeader( Mail->ID, Header );
						Header.Empty();
						if( State == IDLE )
						{
							DialogBox->SetHeader( "Waiting for your next command, Oh Master!");
							DialogBox->m_Count.Format( "%d of %d header(s) received",NoOfMessages, NoOfMessages);
							DialogBox->UpdateData( FALSE );
//							AfxMessageBox( "All headers downloaded.");
						}
						else
						{
							DialogBox->m_Count.Format( "%d of %d header(s) received",NextHeaderNo - 1,NoOfMessages);
							DialogBox->UpdateData( FALSE );
							Mail = pMailList->GetMail( NextHeaderNo );
						}
					}
					else
					{

						if(( NextDataLine[0] == 'S' || NextDataLine[0] == 's' )&& !NextDataLine.Left(7).CompareNoCase( "Subject" ) )
						{
							
							Mail->Subject = NextDataLine.Right( NextDataLine.GetLength() - NextDataLine.Find(':') - 1 );
							Mail->Subject = Mail->Subject.Left(Mail->Subject.GetLength()-2);
						}
						else if( ( NextDataLine[0] == 'D' || NextDataLine[0] == 'd' ) && !NextDataLine.Left(4).CompareNoCase("Date" ) )
						{
							Mail->Date = NextDataLine.Right( NextDataLine.GetLength() - NextDataLine.Find(':') - 1 );
							Mail->Date = Mail->Date.Left(Mail->Date.GetLength()-2);
						}
						else if( (NextDataLine[0] == 'F' || NextDataLine[0] == 'f' ) && !NextDataLine.Left(4).CompareNoCase("From" ) )
						{
							Mail->From = NextDataLine.Right( NextDataLine.GetLength() - NextDataLine.Find(':') - 1 );
							Mail->From = Mail->From.Left(Mail->From.GetLength()-2);
						}
						else if( (NextDataLine[0] == 'T' || NextDataLine[0] == 't' ) &&  (NextDataLine[1] == 'O' || NextDataLine[1] == 'o' ) )
						{
							Mail->To = NextDataLine.Right( NextDataLine.GetLength() - NextDataLine.Find(':') - 1 );
							Mail->To = Mail->To.Left(Mail->To.GetLength()-2);
						}
					}
				}
				else if (LastCommand == DELE )
				{
					
					BYTE Response = IsOK( NextDataLine ) ;// Do nothing
					DialogBox->SetHeader( "Idling");
					State = IDLE;
					if( Response == 1 )
						pMailList->OnMailDelete( DeletionRequestNo, 1 );
					else
						pMailList->OnMailDelete( DeletionRequestNo, 0 );
				}
				else if (LastCommand == RETR )
				{
					if( IsOK( NextDataLine ) == 1);// Do nothing
					else if( Mail->pFilePointer )
						fwrite((LPCSTR) NextDataLine, NextDataLine.GetLength(), 1, Mail->pFilePointer );
					else
						AfxMessageBox( "Invalid File pointer");
					if( NoOfMessages && Length > 2 && 
						NextDataLine[2] =='\n' && 
						NextDataLine[1] =='\r' && 
						NextDataLine[0] =='.' )
					{
						fclose ( Mail->pFilePointer );
						State = IDLE;
						pMailList->OnMailReceipt( Mail->No );
						MessageBeep( MB_ICONEXCLAMATION );
						
					}
					BytesReceived += NextDataLine.GetLength();
					DialogBox->m_Progress.SetPos( BytesReceived * 100 / Mail->Size );
					DialogBox->SetHeader( "Downloading mail...");
				}
			}	
			else if ( State == UPDATE )
			{
				State = NOTCONNECTED;
				
				//Close();
				ShutDown(receives);
				DataStore.Empty();
				if( !OnlyAuthentication )
				{
					DialogBox->SetHeader("Quitting");
					if( IsWindow( DialogBox->m_hWnd ) )
						DialogBox->ShowWindow( SW_HIDE );
					HeaderList.UpdateHeaders( );
					AfxMessageBox("Done checking mail");
				}
				pMailList->OnPOPQuit();				
			}
		}//end of while (1)
	}//end of if(Arguement)
}

BOOL CPOPSocket::IsConnected()
{
	if( State == NOTCONNECTED )
		return FALSE;
	else
		return TRUE;
}

void CPOPSocket::ProcessHeader(CMail *Mail)
{
	CString Header;
	CString Line;
	HeaderList.GetHeader( Mail->ID, Header );
	if( Header.IsEmpty() )
		return;
	int Length = Header.GetLength();
	for( int i = 0; i < Length; i++ )
	{
		Line += Header[i];
		if( Header[i] == '\r' && Header[i+1] == '\n' )
		{
			i++;
			Line += Header[i];
		
			if( Line[0] == 'S' && !Line.Left(7).CompareNoCase( "Subject" ) )
			{
				Mail->Subject = Line.Right( Line.GetLength() - Line.Find(':') - 1 );
				Mail->Subject = Mail->Subject.Left(Mail->Subject.GetLength()-2);
				Mail->Subject.TrimLeft();
				Mail->Subject.TrimRight();
			}
			else if( Line[0] == 'D' && !Line.Left(4).CompareNoCase("Date" ) )
			{
				Mail->Date = Line.Right( Line.GetLength() - Line.Find(':') - 1 );
				Mail->Date = Mail->Date.Left(Mail->Date.GetLength()-2);
				Mail->Date.TrimLeft();
				Mail->Date.TrimRight();
			}
			else if( Line[0] == 'F' && !Line.Left(4).CompareNoCase("From" ) )
			{
				Mail->From = Line.Right( Line.GetLength() - Line.Find(':') - 1 );
				Mail->From = Mail->From.Left(Mail->From.GetLength()-2);
				Mail->From.TrimLeft();
				Mail->From.TrimRight();
			}
			else if( !Line.Left(2).CompareNoCase("To" ) )
			{
				Mail->To = Line.Right( Line.GetLength() - Line.Find(':') - 1 );
				Mail->To = Mail->To.Left( Mail->To.GetLength()-2);
				Mail->To.TrimLeft();
				Mail->To.TrimRight();
			}
			else if( !Line.Left(2).CompareNoCase("CC" ) )
			{
				Mail->CC = Line.Right( Line.GetLength() - Line.Find(':') - 1 );
				Mail->CC = Mail->CC.Left( Mail->CC.GetLength()-2);
				Mail->CC.TrimLeft();
				Mail->CC.TrimRight();
			}
			Line.Empty();
		}		
	}
	pMailList->OnHeaderReceipt( Mail->No );
}

DWORD CPOPSocket::GetNextHeaderNo( DWORD StartNo )
{
	CMail *Mail;
	for( DWORD i = StartNo; ; i++ )
	{
		Mail = pMailList->GetMail( i );
		if( !Mail )
			return 0;
		if( !Mail->Valid )
			continue; //not expected
		if( Mail->From.IsEmpty() && Mail->To.IsEmpty() )
			return Mail->No;
	}
	return 0;
}

BYTE CPOPSocket::IsOK(CString Response)
{
	if( Response[0] == '+' && ( Response[1] == 'O' || Response[1] == 'o' ) && ( Response[2] == 'K' || Response[2] == 'k' ) )
		return 1;
	else if ( Response[0] == '-' && ( Response[1] == 'E' || Response[1] == 'e' ) && ( Response[2] == 'R' || Response[2] == 'r' )  && ( Response[3] == 'R' || Response[3] == 'r' ) )
		return 0;
	else
		return 2;
}

BOOL CPOPSocket::IsAuthenticated()
{
	if( State == TRANSACTION || State == IDLE )
		return TRUE;
	else
		return FALSE;
}

BOOL CPOPSocket::IsQuiting()
{
	if( State == UPDATE )
		return TRUE;
	else
		return FALSE;
}

void CPOPSocket::DonotQuit()
{
	DontQuit = TRUE;
}

void CPOPSocket::CanQuit()
{
	DontQuit = FALSE;
	if( WantToQuit || OnlyAuthentication )
		Quit();
}

void CPOPSocket::Quit()
{
	if( DontQuit )
	{
		WantToQuit = TRUE;
		DialogBox->SetHeader( "Waiting for the SMTP socket to get authentication");
	}
	else if( State != NOTCONNECTED )
	{
		if( State == TRANSACTION && LastCommand == RETR )
		{
			fclose ( Mail->pFilePointer );
			_unlink( Mail->FileName ); 
		}
		CString Message;
		State = UPDATE;
		Message = "QUIT\r\n";
		//::MessageBox(NULL, "Can I quit?", "Question", MB_OK );
		Send( (LPCSTR) Message, Message.GetLength() );
		Message = "C:" + Message;
		DialogBox->AddToStatus( Message );
		DialogBox->SetHeader( "Quiting");
		//	HeaderList.UpdateHeaders();		
	}
}