// TreePane.cpp : implementation file
//

#include "stdafx.h"
#include "Postman.h"
#include "TreePane.h"
#include "ListPane.h"
#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CPostmanApp theApp;
/////////////////////////////////////////////////////////////////////////////
// CTreePane

IMPLEMENT_DYNCREATE(CTreePane, CTreeView)

CTreePane::CTreePane()
{
	SelectionFolder = UNKNOWN;
}

CTreePane::~CTreePane()
{
}


BEGIN_MESSAGE_MAP(CTreePane, CTreeView)
	//{{AFX_MSG_MAP(CTreePane)
	ON_NOTIFY_REFLECT(TVN_SELCHANGED, OnSelchanged)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
	
/////////////////////////////////////////////////////////////////////////////
// CTreePane drawing

void CTreePane::OnDraw(CDC* pDC)
{
	CDocument* pDoc = GetDocument();
	// TODO: add draw code here
}

/////////////////////////////////////////////////////////////////////////////
// CTreePane diagnostics

#ifdef _DEBUG
void CTreePane::AssertValid() const
{
	CTreeView::AssertValid();
}

void CTreePane::Dump(CDumpContext& dc) const
{
	CTreeView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTreePane message handlers

void CTreePane::OnInitialUpdate() 
{
	CTreeView::OnInitialUpdate();
	CTreeCtrl& TreeCtrl = GetTreeCtrl( );

	ImageList.Create( 16, 16, ILC_COLOR16, 5,1);
	ImageList.SetBkColor( RGB(255,255,255) );
	
	
	ImageList.Add( AfxGetApp()->LoadIcon( IDI_LOCALFOLDERS ) );
	ImageList.Add( AfxGetApp()->LoadIcon( IDI_INBOX ) );
	ImageList.Add( AfxGetApp()->LoadIcon( IDI_OUTBOX ) );
	ImageList.Add( AfxGetApp()->LoadIcon( IDI_MAILSERVER ) );
	ImageList.Add( AfxGetApp()->LoadIcon( IDI_DELETED ) );	
	ImageList.Add( AfxGetApp()->LoadIcon( IDI_SENT_FOLDER ) );
	ImageList.Add( AfxGetApp()->LoadIcon( IDR_MAINFRAME ) );
	TreeCtrl.SetImageList( &ImageList , TVSIL_NORMAL );

	hItemPostman = TreeCtrl.InsertItem("Postman");
	hItemLocal = TreeCtrl.InsertItem("Local Folders", hItemPostman);
	hItemInbox = TreeCtrl.InsertItem("InBox",hItemLocal);
	hItemOutbox = TreeCtrl.InsertItem("OutBox",hItemLocal);
	hItemSentTo = TreeCtrl.InsertItem("Sent Items",hItemLocal);
	hItemDeleted = TreeCtrl.InsertItem("Deleted Items",hItemLocal);
	hItemMailServer = TreeCtrl.InsertItem("Mail Server", hItemPostman);
	
	TreeCtrl.SetItemImage( hItemInbox, 1 , 1 );
	TreeCtrl.SetItemImage( hItemOutbox, 2 , 2 );
	TreeCtrl.SetItemImage( hItemMailServer, 3 , 3 );
	TreeCtrl.SetItemImage( hItemDeleted, 4 , 4 );
	TreeCtrl.SetItemImage( hItemSentTo, 5 , 5 );
	TreeCtrl.SetItemImage( hItemPostman, 6 , 6 );

	TreeCtrl.Expand( hItemPostman, TVE_EXPAND );
	TreeCtrl.Expand( hItemLocal, TVE_EXPAND );
	TreeCtrl.Expand( hItemInbox, TVE_EXPAND );
	TreeCtrl.Select( hItemInbox,TVGN_CARET );
}

BOOL CTreePane::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext) 
{
	return CWnd::Create(lpszClassName, lpszWindowName, dwStyle|TVS_HASLINES|TVS_LINESATROOT|TVS_HASBUTTONS, rect, pParentWnd, nID, pContext);
}

void CTreePane::OnSelchanged(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	CWinApp *App = AfxGetApp();
	CMainFrame *MainWnd = (CMainFrame *)App->GetMainWnd();
	CMailList *pMailList;

	CString Folder = GetTreeCtrl( ).GetItemText( pNMTreeView->itemNew.hItem );
	if( Folder.CompareNoCase( "InBox" ) == 0 )
	{
		pMailList = MainWnd->pInBox;
		SelectionFolder = INBOX;

	}
	else if( Folder.CompareNoCase( "OutBox" ) == 0 )
	{
		pMailList = MainWnd->pOutBox;
		SelectionFolder = OUTBOX;
	}
	else if( Folder.CompareNoCase( "Mail Server" ) == 0 )
	{
		pMailList = MainWnd->pMailList;
		SelectionFolder = MAILSERVER;
	}
	else if( Folder.CompareNoCase( "Deleted Items" ) == 0 )
	{
		pMailList = MainWnd->pDeleted;
		SelectionFolder = DELETED;
	}
	else if( Folder.CompareNoCase( "Sent Items" ) == 0 )
	{
		pMailList = MainWnd->pSentTo;
		SelectionFolder = SENTTO;
	}
	else
	{
		SelectionFolder = UNKNOWN;
		pMailList = NULL;
	}
	CString Text = "Postman - ";
	Text = Text + Folder;
	MainWnd->SetWindowText( Text );
	UpdateList( pMailList);
	*pResult = 0;
}

void CTreePane::UpdateList( CMailList *pMailList)
{
	CMail *Mail;
	CString Size, Date;
	CTime Time;
	DWORD S;
	LVITEM Item;
	CWinApp *App = AfxGetApp();
	CMainFrame *MainWnd = (CMainFrame *)App->GetMainWnd();
	CListPane *ListPane = ( CListPane *) MainWnd->m_wndSplitter.GetPane(0,1);
	CListCtrl &MailListCtrl = ListPane->GetListCtrl();
	
	MailListCtrl.DeleteAllItems();
	Item.mask = LVIF_IMAGE;
	Item.iSubItem = 0;
	if( !pMailList )
		return;
	
	int i, j ;
	for( i = 0, j = 1; ; j++ )
	{
		Item.iImage = pMailList->ImageIdx;
		Mail = pMailList->GetMail(j);
		if( !Mail )
			break;
		if( !Mail->Valid )
			continue;
		if( SelectionFolder == MAILSERVER && !Mail->Downloaded ) 
		{
			Item.iImage = -1;
		}
		if( Mail->Date.IsEmpty() && Mail->From.IsEmpty() && Mail->Subject.IsEmpty() )
			continue;  // Partial header; ignoring
		if( Mail->Size < 1024 )
		{
			Size.Format("%ld B",Mail->Size );
		}
		else if ( Mail->Size < 1048576 )
		{
			S=(Mail->Size + 512 ) / 1024;
			Size.Format("%ld KB", S);
		}
		else
		{
			S=(Mail->Size + 524288 ) / 1048576 ;
			Size.Format("%ld MB", S);
		}
		if( !Mail->Date.IsEmpty() )
		{
			Time = CPostmanApp::ConvertDate( Mail->Date );
			Date.Format( "%d-%d-%d %0.2d:%0.2d", Time.GetDay(), Time.GetMonth(), Time.GetYear(), Time.GetHour(), Time.GetMinute() );
		}
		
		MailListCtrl.InsertItem ( i , Mail->From );
		Item.iItem = i ;
		MailListCtrl.SetItem( &Item );
		MailListCtrl.SetItemText( i , 1, Mail->To);
		MailListCtrl.SetItemText( i , 2, Mail->Subject);
		MailListCtrl.SetItemText( i , 3, Date);
		MailListCtrl.SetItemText( i , 4, Size);
		MailListCtrl.SetItemData( i , j );
		i++;
	}
	MailListCtrl.SortItems( CListPane::SortDate, (LPARAM)&MailListCtrl);
}

void CTreePane::SetSelection(BYTE Folder)
{
	CTreeCtrl& TreeCtrl = GetTreeCtrl( );
	if( Folder == INBOX || Folder == OUTBOX || Folder == MAILSERVER || Folder == SENTTO || Folder == DELETED )
	{
		TreeCtrl.Expand( hItemPostman, TVE_EXPAND );
		TreeCtrl.Expand( hItemLocal, TVE_EXPAND );
		if( Folder == INBOX )
		{
			TreeCtrl.Expand( hItemInbox, TVE_EXPAND );
			TreeCtrl.Select( hItemInbox,TVGN_CARET );
		}
		else if( Folder == OUTBOX )
		{
			TreeCtrl.Expand( hItemOutbox, TVE_EXPAND );
			TreeCtrl.Select( hItemOutbox,TVGN_CARET );
		}
		else if( Folder == MAILSERVER)
		{
			TreeCtrl.Expand( hItemMailServer, TVE_EXPAND );
			TreeCtrl.Select( hItemMailServer,TVGN_CARET );
		}
		else if( Folder == SENTTO)
		{
			TreeCtrl.Expand( hItemSentTo, TVE_EXPAND );
			TreeCtrl.Select( hItemSentTo,TVGN_CARET );
		}
		else if( Folder == DELETED)
		{
			TreeCtrl.Expand( hItemDeleted, TVE_EXPAND );
			TreeCtrl.Select( hItemDeleted,TVGN_CARET );
		}
	}	
}